@@ -5,6 +5,7 @@
 #pragma once
 
 #include <array>
+#include <concepts>
 #include <utility>
 #include <vector>
 
@@ -16,7 +17,7 @@ namespace basix::cell
 {
 
 /// Cell type
-enum class type
+enum class type : int
 {
   point = 0,
   interval = 1,
@@ -33,7 +34,8 @@ enum class type
 /// @return (0) Vertex point data of the cell and (1) the shape of the
 /// data array. The points are stored in row-major format and the shape
 /// is is (npoints, gdim)
-std::pair<std::vector<double>, std::array<std::size_t, 2>>
+template <std::floating_point T>
+std::pair<std::vector<T>, std::array<std::size_t, 2>>
 geometry(cell::type celltype);
 
 /// Cell topology
@@ -59,13 +61,11 @@ sub_entity_connectivity(cell::type celltype);
 /// @param dim Dimension of sub-entity
 /// @param index Local index of sub-entity
 /// @return Set of vertex points of the sub-entity. Shape is (npoints, gdim)
-std::pair<std::vector<double>, std::array<std::size_t, 2>>
+template <std::floating_point T>
+std::pair<std::vector<T>, std::array<std::size_t, 2>>
 sub_entity_geometry(cell::type celltype, int dim, int index);
 
 /// Number of sub-entities of a cell by topological dimension
-/// @todo Optimise this function
-/// @warning This function is expensive to call. Do not use in
-/// performance critical code
 /// @param celltype The cell::type
 /// @param dim Dimension of sub-entity
 /// @return The number of sub-entities of the given dimension
@@ -86,19 +86,22 @@ cell::type sub_entity_type(cell::type celltype, int dim, int index);
 /// Get the volume of a reference cell
 /// @param cell_type Type of cell
 /// @return The volume of the cell
-double volume(cell::type cell_type);
+template <std::floating_point T>
+T volume(cell::type cell_type);
 
 /// Get the (outward) normals to the facets of a reference cell
 /// @param cell_type Type of cell
 /// @return The outward normals. Shape is (nfacets, gdim)
-std::pair<std::vector<double>, std::array<std::size_t, 2>>
+template <std::floating_point T>
+std::pair<std::vector<T>, std::array<std::size_t, 2>>
 facet_outward_normals(cell::type cell_type);
 
 /// Get the normals to the facets of a reference cell oriented using the
 /// low-to-high ordering of the facet
 /// @param cell_type Type of cell
 /// @return The normals. Shape is (nfacets, gdim)
-std::pair<std::vector<double>, std::array<std::size_t, 2>>
+template <std::floating_point T>
+std::pair<std::vector<T>, std::array<std::size_t, 2>>
 facet_normals(cell::type cell_type);
 
 /// Get an array of bools indicating whether or not the facet normals are
@@ -110,8 +113,8 @@ std::vector<bool> facet_orientations(cell::type cell_type);
 /// Get the reference volumes of the facets of a reference cell
 /// @param cell_type Type of cell
 /// @return The volumes of the references associated with each facet
-// FIXME: This could be a std::vector<double>
-std::vector<double> facet_reference_volumes(cell::type cell_type);
+template <std::floating_point T>
+std::vector<T> facet_reference_volumes(cell::type cell_type);
 
 /// Get the types of the subentities of a reference cell
 /// @param cell_type Type of cell
@@ -121,7 +124,8 @@ std::vector<std::vector<cell::type>> subentity_types(cell::type cell_type);
 /// Get the jacobians of the facets of a reference cell
 /// @param cell_type Type of cell
 /// @return The jacobians of the facets. Shape is (nfacets, gdim, gdim - 1)
-std::pair<std::vector<double>, std::array<std::size_t, 3>>
+template <std::floating_point T>
+std::pair<std::vector<T>, std::array<std::size_t, 3>>
 facet_jacobians(cell::type cell_type);
 
 } // namespace basix::cell
