@@ -7,6 +7,7 @@
 #include "cell.h"
 #include "element-families.h"
 #include "finite-element.h"
+#include <concepts>
 
 namespace basix::element
 {
@@ -18,7 +19,8 @@ namespace basix::element
 /// @param[in] discontinuous Controls whether the element is continuous or
 /// discontinuous
 /// @return A finite element
-FiniteElement create_bdm(cell::type celltype, int degree,
-                         lagrange_variant lvariant, bool discontinuous);
+template <std::floating_point T>
+FiniteElement<T> create_bdm(cell::type celltype, int degree,
+                            lagrange_variant lvariant, bool discontinuous);
 
 } // namespace basix::element
