@@ -7,13 +7,14 @@
 #include "cell.h"
 #include "mdspan.hpp"
 #include <array>
+#include <concepts>
 #include <utility>
 #include <vector>
 
 /// Polynomials
 namespace basix::polynomials
 {
-/// Variants of a Lagrange space that can be created
+/// @brief Variants of a Lagrange space that can be created.
 enum class type
 {
   legendre = 0,
@@ -28,18 +29,19 @@ enum class type
 /// (number of points, geometric dimension).
 /// @return Polynomial sets, for each derivative, tabulated at points.
 /// The shape is `(basis index, number of points)`.
-std::pair<std::vector<double>, std::array<std::size_t, 2>>
+template <std::floating_point T>
+std::pair<std::vector<T>, std::array<std::size_t, 2>>
 tabulate(polynomials::type polytype, cell::type celltype, int d,
-         std::experimental::mdspan<const double,
-                                   std::experimental::dextents<std::size_t, 2>>
+         MDSPAN_IMPL_STANDARD_NAMESPACE::mdspan<
+             const T, MDSPAN_IMPL_STANDARD_NAMESPACE::dextents<std::size_t, 2>>
              x);
 
 /// @brief Dimension of a polynomial space.
-/// @param[in] polytype The polynomial type
-/// @param[in] cell The cell type
-/// @param[in] d The polynomial degree
+/// @param[in] polytype Polynomial type
+/// @param[in] cell Cell type
+/// @param[in] d Polynomial degree
 /// @return The number terms in the basis spanning a space of
-/// polynomial degree @p d
+/// polynomial degree `d`.
 int dim(polynomials::type polytype, cell::type cell, int d);
 
 } // namespace basix::polynomials
