@@ -7,12 +7,12 @@
 #
 # First, we import Basix and Numpy.
 
-import basix
-import numpy as np
-from basix import CellType, MapType, PolynomialType, LatticeType
-
-from mpl_toolkits import mplot3d  # noqa: F401
 import matplotlib.pyplot as plt
+import numpy as np
+from mpl_toolkits import mplot3d  # noqa: F401
+
+import basix
+from basix import CellType, LatticeType, MapType, PolynomialType, PolysetType, SobolevSpace
 
 # Conforming CR element on a triangle
 # ===================================
@@ -29,7 +29,6 @@ import matplotlib.pyplot as plt
 
 
 def create_ccr_triangle(degree):
-
     if degree == 1:
         wcoeffs = np.eye(3)
 
@@ -44,34 +43,43 @@ def create_ccr_triangle(degree):
 
         M = [[], [], [], []]
         for _ in range(3):
-            M[0].append(np.array([[[1.]]]))
+            M[0].append(np.array([[[1.0]]]))
 
         for _ in range(3):
             M[1].append(np.zeros((0, 1, 0)))
         M[2].append(np.zeros((0, 1, 0, 1)))
 
         return basix.create_custom_element(
-            CellType.triangle, [], wcoeffs, x, M, 0, MapType.identity, False, 1, 1)
+            CellType.triangle,
+            [],
+            wcoeffs,
+            x,
+            M,
+            0,
+            MapType.identity,
+            SobolevSpace.L2,
+            False,
+            1,
+            1,
+            PolysetType.standard,
+        )
 
     npoly = (degree + 2) * (degree + 3) // 2
     ndofs = degree * (degree + 5) // 2
     wcoeffs = np.zeros((ndofs, npoly))
 
-    dof_n = 0
-    for i in range((degree + 1) * (degree + 2) // 2):
-        wcoeffs[dof_n, dof_n] = 1
-        dof_n += 1
+    r = (degree + 1) * (degree + 2) // 2
+    wcoeffs[:r, :r] = np.eye(r)
 
     pts, wts = basix.make_quadrature(CellType.triangle, 2 * (degree + 1))
     poly = basix.tabulate_polynomials(PolynomialType.legendre, CellType.triangle, degree + 1, pts)
+    x = pts[:, 0]
+    y = pts[:, 1]
     for i in range(1, degree):
-        x = pts[:, 0]
-        y = pts[:, 1]
-        f = x ** i * y ** (degree - i) * (x + y)
-
+        f = x**i * y ** (degree - i) * (x + y)
         for j in range(npoly):
-            wcoeffs[dof_n, j] = sum(f * poly[j, :] * wts)
-        dof_n += 1
+            wcoeffs[r, j] = sum(f * poly[j, :] * wts)
+        r += 1
 
     geometry = basix.geometry(CellType.triangle)
     topology = basix.topology(CellType.triangle)
@@ -79,16 +87,14 @@ def create_ccr_triangle(degree):
     M = [[], [], [], []]
     for v in topology[0]:
         x[0].append(np.array(geometry[v]))
-        M[0].append(np.array([[[[1.]]]]))
+        M[0].append(np.array([[[[1.0]]]]))
     pts = basix.create_lattice(CellType.interval, degree, LatticeType.equispaced, False)
     mat = np.zeros((len(pts), 1, len(pts), 1))
     mat[:, 0, :, 0] = np.eye(len(pts))
     for e in topology[1]:
-        edge_pts = []
         v0 = geometry[e[0]]
         v1 = geometry[e[1]]
-        for p in pts:
-            edge_pts.append(v0 + p * (v1 - v0))
+        edge_pts = [v0 + p * (v1 - v0) for p in pts]
         x[1].append(np.array(edge_pts))
         M[1].append(mat)
     pts = basix.create_lattice(CellType.triangle, degree + 1, LatticeType.equispaced, False)
@@ -98,7 +104,19 @@ def create_ccr_triangle(degree):
     M[2].append(mat)
 
     return basix.create_custom_element(
-        CellType.triangle, [], wcoeffs, x, M, 0, MapType.identity, False, degree, degree + 1)
+        CellType.triangle,
+        [],
+        wcoeffs,
+        x,
+        M,
+        0,
+        MapType.identity,
+        SobolevSpace.L2,
+        False,
+        degree,
+        degree + 1,
+        PolysetType.standard,
+    )
 
 
 # We can then create a degree 2 conforming CR element.
@@ -117,9 +135,9 @@ fig = plt.figure(figsize=(8, 8))
 
 for n in range(7):
     if n == 6:
-        ax = plt.subplot(3, 3, n + 2, projection='3d')
+        ax = plt.subplot(3, 3, n + 2, projection="3d")
     else:
-        ax = plt.subplot(3, 3, n + 1, projection='3d')
+        ax = plt.subplot(3, 3, n + 1, projection="3d")
     ax.plot([0, 1, 0, 0], [0, 0, 1, 0], [0, 0, 0, 0], "k-")
     ax.scatter(x, y, z[:, n, 0])
 
@@ -142,7 +160,7 @@ z = e.tabulate(0, pts)[0]
 fig = plt.figure(figsize=(11, 8))
 
 for n in range(12):
-    ax = plt.subplot(3, 4, n + 1, projection='3d')
+    ax = plt.subplot(3, 4, n + 1, projection="3d")
     ax.plot([0, 1, 0, 0], [0, 0, 1, 0], [0, 0, 0, 0], "k-")
     ax.scatter(x, y, z[:, n, 0])
 
