@@ -2,9 +2,10 @@
 # FEniCS Project
 # SPDX-License-Identifier: MIT
 
-import basix
 import pytest
 
+import basix
+
 cells = [
     basix.CellType.point,
     basix.CellType.interval,
@@ -29,6 +30,7 @@ elements = [
     basix.ElementFamily.DPC,
     basix.ElementFamily.CR,
     basix.ElementFamily.Hermite,
+    basix.ElementFamily.iso,
     basix.ElementFamily.custom,
 ]
 
@@ -37,28 +39,20 @@ variants = [
     [basix.LagrangeVariant.gll_warped],
     [basix.LagrangeVariant.legendre],
     [basix.LagrangeVariant.bernstein],
-    [basix.DPCVariant.diagonal_gll],
-    [basix.DPCVariant.legendre],
+    [basix.LagrangeVariant.unset, basix.DPCVariant.diagonal_gll],
+    [basix.LagrangeVariant.unset, basix.DPCVariant.legendre],
     [basix.LagrangeVariant.legendre, basix.DPCVariant.legendre],
 ]
 
 
 def test_all_cells_included():
-    all_cells = set()
-    for c in dir(basix.CellType):
-        if not c.startswith("_") and c not in ["name", "value"]:
-            all_cells.add(getattr(basix.CellType, c))
-
-    assert all_cells == set(cells)
+    all_cells = [cell for cell in basix.CellType]
+    assert sorted(all_cells) == sorted(cells)
 
 
 def test_all_elements_included():
-    all_elements = set()
-    for c in dir(basix.ElementFamily):
-        if not c.startswith("_") and c not in ["name", "value"]:
-            all_elements.add(getattr(basix.ElementFamily, c))
-
-    assert all_elements == set(elements)
+    all_elements = [e for e in basix.ElementFamily]
+    assert sorted(all_elements) == sorted(elements)
 
 
 @pytest.mark.parametrize("cell", cells)
@@ -76,8 +70,14 @@ def test_create_element(cell, degree, family, variant):
             raise e
 
     try:
-        element = basix.create_element(family, cell, degree, *variant, True)
+        element = basix.create_element(family, cell, degree, *variant, discontinuous=True)
         assert element.degree == degree
     except RuntimeError as e:
         if len(e.args) == 0 or "dgesv" in e.args[0]:
             raise e
+
+
+def test_create_high_degree_lagrange():
+    basix.create_element(
+        basix.ElementFamily.P, basix.CellType.hexahedron, 7, basix.LagrangeVariant.gll_isaac
+    )
