@@ -2,21 +2,27 @@
 # FEniCS Project
 # SPDX-License-Identifier: MIT
 
-import basix
-import pytest
-import numpy as np
 import random
+
+import numpy as np
+import pytest
+
+import basix
+
 from .utils import parametrize_over_elements
 
 
 @parametrize_over_elements(3)
 def test_if_permutations(cell_type, element_type, degree, element_args):
     e = basix.create_element(element_type, cell_type, degree, *element_args)
-
     for t in e.base_transformations():
         for row in t:
             a = np.argmax(row)
-            if not np.isclose(row[a], 1) or not np.allclose(row[:a], 0) or not np.allclose(row[a + 1:], 0):
+            if (
+                not np.isclose(row[a], 1)
+                or not np.allclose(row[:a], 0)
+                or not np.allclose(row[a + 1 :], 0)
+            ):
                 assert not e.dof_transformations_are_permutations
                 return
     assert e.dof_transformations_are_permutations
@@ -25,7 +31,6 @@ def test_if_permutations(cell_type, element_type, degree, element_args):
 @parametrize_over_elements(3)
 def test_if_identity(cell_type, element_type, degree, element_args):
     e = basix.create_element(element_type, cell_type, degree, *element_args)
-
     for t in e.base_transformations():
         if not np.allclose(t, np.eye(t.shape[0])):
             assert not e.dof_transformations_are_identity
@@ -42,22 +47,20 @@ def test_non_zero(cell_type, element_type, degree, element_args):
 
 
 @parametrize_over_elements(5)
-def test_apply_to_transpose(cell_type, element_type, degree, element_args):
+def test_apply_right(cell_type, element_type, degree, element_args):
     random.seed(42)
-
     e = basix.create_element(element_type, cell_type, degree, *element_args)
     size = e.dim
-
     for i in range(10):
         cell_info = random.randrange(2**30)
 
         data1 = np.array(list(range(size**2)), dtype=np.float32)
-        data1 = e.apply_dof_transformation(data1, size, cell_info)
+        e.T_apply(data1, size, cell_info)
         data1 = data1.reshape((size, size))
 
         # This is the transpose of the data used above
         data2 = np.array([size * j + i for i in range(size) for j in range(size)], dtype=np.float32)
-        data2 = e.apply_dof_transformation_to_transpose(data2, size, cell_info)
+        e.Tt_apply_right(data2, size, cell_info)
         data2 = data2.reshape((size, size))
 
         assert np.allclose(data1.transpose(), data2)
@@ -76,9 +79,7 @@ def test_triangle_transformation_degrees(element_type, degree, element_args):
     assert len(bt) == 3
     identity = np.identity(e.dim)
     for i, degree in enumerate([2, 2, 2]):
-        assert np.allclose(
-            np.linalg.matrix_power(bt[i], degree),
-            identity)
+        assert np.allclose(np.linalg.matrix_power(bt[i], degree), identity)
 
 
 @parametrize_over_elements(5, basix.CellType.tetrahedron)
@@ -88,9 +89,7 @@ def test_tetrahedron_transformation_degrees(element_type, degree, element_args):
     assert len(bt) == 14
     identity = np.identity(e.dim)
     for i, degree in enumerate([2, 2, 2, 2, 2, 2, 3, 2, 3, 2, 3, 2, 3, 2]):
-        assert np.allclose(
-            np.linalg.matrix_power(bt[i], degree),
-            identity)
+        assert np.allclose(np.linalg.matrix_power(bt[i], degree), identity)
 
 
 @parametrize_over_elements(5, basix.CellType.quadrilateral)
@@ -100,9 +99,7 @@ def test_quadrilateral_transformation_degrees(element_type, degree, element_args
     assert len(bt) == 4
     identity = np.identity(e.dim)
     for i, degree in enumerate([2, 2, 2, 2]):
-        assert np.allclose(
-            np.linalg.matrix_power(bt[i], degree),
-            identity)
+        assert np.allclose(np.linalg.matrix_power(bt[i], degree), identity)
 
 
 @parametrize_over_elements(5, basix.CellType.hexahedron)
@@ -111,12 +108,11 @@ def test_hexahedron_transformation_degrees(element_type, degree, element_args):
     bt = e.base_transformations()
     assert len(bt) == 24
     identity = np.identity(e.dim)
-    for i, degree in enumerate([2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
-                                4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2]):
+    for i, degree in enumerate(
+        [2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2]
+    ):
         # TODO: remove the atol here once non-equispaced variants are implemented
-        assert np.allclose(
-            np.linalg.matrix_power(bt[i], degree),
-            identity, atol=1e-6)
+        assert np.allclose(np.linalg.matrix_power(bt[i], degree), identity, atol=1e-6)
 
 
 @parametrize_over_elements(5, basix.CellType.prism)
@@ -125,11 +121,8 @@ def test_prism_transformation_degrees(element_type, degree, element_args):
     bt = e.base_transformations()
     assert len(bt) == 19
     identity = np.identity(e.dim)
-    for i, degree in enumerate([2, 2, 2, 2, 2, 2, 2, 2, 2,
-                                3, 2, 4, 2, 4, 2, 4, 2, 3, 2]):
-        assert np.allclose(
-            np.linalg.matrix_power(bt[i], degree),
-            identity)
+    for i, degree in enumerate([2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 2, 4, 2, 4, 2, 4, 2, 3, 2]):
+        assert np.allclose(np.linalg.matrix_power(bt[i], degree), identity)
 
 
 @parametrize_over_elements(5, basix.CellType.pyramid)
@@ -138,11 +131,8 @@ def test_pyramid_transformation_degrees(element_type, degree, element_args):
     bt = e.base_transformations()
     assert len(bt) == 18
     identity = np.identity(e.dim)
-    for i, degree in enumerate([2, 2, 2, 2, 2, 2, 2, 2,
-                                4, 2, 3, 2, 3, 2, 3, 2, 3, 2]):
-        assert np.allclose(
-            np.linalg.matrix_power(bt[i], degree),
-            identity)
+    for i, degree in enumerate([2, 2, 2, 2, 2, 2, 2, 2, 4, 2, 3, 2, 3, 2, 3, 2, 3, 2]):
+        assert np.allclose(np.linalg.matrix_power(bt[i], degree), identity)
 
 
 @parametrize_over_elements(5, basix.CellType.triangle)
@@ -166,13 +156,13 @@ def test_transformation_of_tabulated_data_triangle(element_type, degree, element
         detJ = np.array([np.linalg.det(_J) for p in points])
         K = np.array([np.linalg.inv(_J) for p in points])
         mapped_values = e.push_forward(reflected_values, J, detJ, K)
-
         for i, j in zip(values, mapped_values):
             for d in range(e.value_size):
                 i_slice = i[:, d]
                 j_slice = j[:, d]
-                assert np.allclose((bt[0].dot(i_slice))[start: start + ndofs],
-                                   j_slice[start: start + ndofs])
+                assert np.allclose(
+                    (bt[0].dot(i_slice))[start : start + ndofs], j_slice[start : start + ndofs]
+                )
 
 
 @parametrize_over_elements(5, basix.CellType.quadrilateral)
@@ -200,8 +190,9 @@ def test_transformation_of_tabulated_data_quadrilateral(element_type, degree, el
             for d in range(e.value_size):
                 i_slice = i[:, d]
                 j_slice = j[:, d]
-                assert np.allclose((bt[0].dot(i_slice))[start: start + ndofs],
-                                   j_slice[start: start + ndofs])
+                assert np.allclose(
+                    (bt[0].dot(i_slice))[start : start + ndofs], j_slice[start : start + ndofs]
+                )
 
 
 @parametrize_over_elements(5, basix.CellType.tetrahedron)
@@ -210,17 +201,23 @@ def test_transformation_of_tabulated_data_tetrahedron(element_type, degree, elem
     bt = e.base_transformations()
 
     N = 4
-    points = np.array([[i / N, j / N, k / N]
-                       for i in range(N + 1) for j in range(N + 1 - i) for k in range(N + 1 - i - j)])
+    points = np.array(
+        [
+            [i / N, j / N, k / N]
+            for i in range(N + 1)
+            for j in range(N + 1 - i)
+            for k in range(N + 1 - i - j)
+        ]
+    )
     values = e.tabulate(0, points)[0]
 
     start = sum(e.num_entity_dofs[0])
     ndofs = e.num_entity_dofs[1][0]
     if ndofs != 0:
-        # Check that the 0th transformation undoes the effect of reflecting edge 0
+        # Check that the 0th transformation undoes the effect of
+        # reflecting edge 0
         reflected_points = np.array([[p[0], p[2], p[1]] for p in points])
         reflected_values = e.tabulate(0, reflected_points)[0]
-
         _J = np.array([[1, 0, 0], [0, 0, 1], [0, 1, 0]])
         J = np.array([_J for p in points])
         detJ = np.array([np.linalg.det(_J) for p in points])
@@ -230,13 +227,15 @@ def test_transformation_of_tabulated_data_tetrahedron(element_type, degree, elem
             for d in range(e.value_size):
                 i_slice = i[:, d]
                 j_slice = j[:, d]
-                assert np.allclose((bt[0].dot(i_slice))[start: start + ndofs],
-                                   j_slice[start: start + ndofs])
+                assert np.allclose(
+                    (bt[0].dot(i_slice))[start : start + ndofs], j_slice[start : start + ndofs]
+                )
 
     start = sum(e.num_entity_dofs[0]) + sum(e.num_entity_dofs[1])
     ndofs = e.num_entity_dofs[2][0]
     if ndofs != 0:
-        # Check that the 6th transformation undoes the effect of rotating face 0
+        # Check that the 6th transformation undoes the effect of
+        # rotating face 0
         rotated_points = np.array([[p[2], p[0], p[1]] for p in points])
         rotated_values = e.tabulate(0, rotated_points)[0]
 
@@ -249,14 +248,14 @@ def test_transformation_of_tabulated_data_tetrahedron(element_type, degree, elem
             for d in range(e.value_size):
                 i_slice = i[:, d]
                 j_slice = j[:, d]
-                assert np.allclose(bt[6].dot(i_slice)[start: start + ndofs],
-                                   j_slice[start: start + ndofs])
+                assert np.allclose(
+                    bt[6].dot(i_slice)[start : start + ndofs], j_slice[start : start + ndofs]
+                )
 
     if ndofs != 0:
         # Check that the 7th transformation undoes the effect of reflecting face 0
         reflected_points = np.array([[p[0], p[2], p[1]] for p in points])
         reflected_values = e.tabulate(0, reflected_points)[0]
-
         _J = np.array([[1, 0, 0], [0, 0, 1], [0, 1, 0]])
         J = np.array([_J for p in points])
         detJ = np.array([np.linalg.det(_J) for p in points])
@@ -266,22 +265,26 @@ def test_transformation_of_tabulated_data_tetrahedron(element_type, degree, elem
             for d in range(e.value_size):
                 i_slice = i[:, d]
                 j_slice = j[:, d]
-                assert np.allclose((bt[7].dot(i_slice))[start: start + ndofs],
-                                   j_slice[start: start + ndofs])
+                assert np.allclose(
+                    (bt[7].dot(i_slice))[start : start + ndofs], j_slice[start : start + ndofs]
+                )
 
 
 @parametrize_over_elements(3, basix.CellType.hexahedron)
 def test_transformation_of_tabulated_data_hexahedron(element_type, degree, element_args):
     if degree > 4 and element_type in [basix.ElementFamily.RT, basix.ElementFamily.N1E]:
-        pytest.xfail("High degree Hdiv and Hcurl spaces on hexes based on "
-                     "Lagrange spaces equally spaced points are unstable.")
+        pytest.xfail(
+            "High degree Hdiv and Hcurl spaces on hexes based on "
+            "Lagrange spaces equally spaced points are unstable."
+        )
 
     e = basix.create_element(element_type, basix.CellType.hexahedron, degree, *element_args)
     bt = e.base_transformations()
 
     N = 4
-    points = np.array([[i / N, j / N, k / N]
-                       for i in range(N + 1) for j in range(N + 1) for k in range(N + 1)])
+    points = np.array(
+        [[i / N, j / N, k / N] for i in range(N + 1) for j in range(N + 1) for k in range(N + 1)]
+    )
     values = e.tabulate(0, points)[0]
 
     start = sum(e.num_entity_dofs[0])
@@ -290,7 +293,6 @@ def test_transformation_of_tabulated_data_hexahedron(element_type, degree, eleme
         # Check that the 0th transformation undoes the effect of reflecting edge 0
         reflected_points = np.array([[1 - p[0], p[1], p[2]] for p in points])
         reflected_values = e.tabulate(0, reflected_points)[0]
-
         _J = np.array([[-1, 0, 0], [0, 1, 0], [0, 0, 1]])
         J = np.array([_J for p in points])
         detJ = np.array([np.linalg.det(_J) for p in points])
@@ -300,16 +302,17 @@ def test_transformation_of_tabulated_data_hexahedron(element_type, degree, eleme
             for d in range(e.value_size):
                 i_slice = i[:, d]
                 j_slice = j[:, d]
-                assert np.allclose((bt[0].dot(i_slice))[start: start + ndofs],
-                                   j_slice[start: start + ndofs])
+                assert np.allclose(
+                    (bt[0].dot(i_slice))[start : start + ndofs], j_slice[start : start + ndofs]
+                )
 
     start = sum(e.num_entity_dofs[0]) + sum(e.num_entity_dofs[1])
     ndofs = e.num_entity_dofs[2][0]
     if ndofs != 0:
-        # Check that the 12th transformation undoes the effect of rotating face 0
+        # Check that the 12th transformation undoes the effect of
+        # rotating face 0
         rotated_points = np.array([[1 - p[1], p[0], p[2]] for p in points])
         rotated_values = e.tabulate(0, rotated_points)[0]
-
         _J = np.array([[0, 1, 0], [-1, 0, 0], [0, 0, 1]])
         J = np.array([_J for p in points])
         detJ = np.array([np.linalg.det(_J) for p in points])
@@ -319,11 +322,13 @@ def test_transformation_of_tabulated_data_hexahedron(element_type, degree, eleme
             for d in range(e.value_size):
                 i_slice = i[:, d]
                 j_slice = j[:, d]
-                assert np.allclose(bt[12].dot(i_slice)[start: start + ndofs],
-                                   j_slice[start: start + ndofs])
+                assert np.allclose(
+                    bt[12].dot(i_slice)[start : start + ndofs], j_slice[start : start + ndofs]
+                )
 
     if ndofs != 0:
-        # Check that the 13th transformation undoes the effect of reflecting face 0
+        # Check that the 13th transformation undoes the effect of
+        # reflecting face 0
         reflected_points = np.array([[p[1], p[0], p[2]] for p in points])
         reflected_values = e.tabulate(0, reflected_points)[0]
 
@@ -336,8 +341,9 @@ def test_transformation_of_tabulated_data_hexahedron(element_type, degree, eleme
             for d in range(e.value_size):
                 i_slice = i[:, d]
                 j_slice = j[:, d]
-                assert np.allclose((bt[13].dot(i_slice))[start: start + ndofs],
-                                   j_slice[start: start + ndofs])
+                assert np.allclose(
+                    (bt[13].dot(i_slice))[start : start + ndofs], j_slice[start : start + ndofs]
+                )
 
 
 @parametrize_over_elements(3, basix.CellType.prism)
@@ -346,17 +352,23 @@ def test_transformation_of_tabulated_data_prism(element_type, degree, element_ar
     bt = e.base_transformations()
 
     N = 4
-    points = np.array([[i / N, j / N, k / N]
-                       for i in range(N + 1) for j in range(N + 1 - i) for k in range(N + 1)])
+    points = np.array(
+        [
+            [i / N, j / N, k / N]
+            for i in range(N + 1)
+            for j in range(N + 1 - i)
+            for k in range(N + 1)
+        ]
+    )
     values = e.tabulate(0, points)[0]
 
     start = sum(e.num_entity_dofs[0])
     ndofs = e.num_entity_dofs[1][0]
     if ndofs != 0:
-        # Check that the 0th transformation undoes the effect of reflecting edge 0
+        # Check that the 0th transformation undoes the effect of
+        # reflecting edge 0
         reflected_points = np.array([[1 - p[1] - p[0], p[1], p[2]] for p in points])
         reflected_values = e.tabulate(0, reflected_points)[0]
-
         _J = np.array([[-1, 0, 0], [-1, 1, 0], [0, 0, 1]])
         J = np.array([_J for p in points])
         detJ = np.array([np.linalg.det(_J) for p in points])
@@ -366,13 +378,15 @@ def test_transformation_of_tabulated_data_prism(element_type, degree, element_ar
             for d in range(e.value_size):
                 i_slice = i[:, d]
                 j_slice = j[:, d]
-                assert np.allclose((bt[0].dot(i_slice))[start: start + ndofs],
-                                   j_slice[start: start + ndofs])
+                assert np.allclose(
+                    (bt[0].dot(i_slice))[start : start + ndofs], j_slice[start : start + ndofs]
+                )
 
     start = sum(e.num_entity_dofs[0]) + sum(e.num_entity_dofs[1])
     ndofs = e.num_entity_dofs[2][0]
     if ndofs != 0:
-        # Check that the 10th transformation undoes the effect of rotating face 0
+        # Check that the 10th transformation undoes the effect of
+        # rotating face 0
         rotated_points = np.array([[1 - p[0] - p[1], p[0], p[2]] for p in points])
         rotated_values = e.tabulate(0, rotated_points)[0]
 
@@ -385,14 +399,15 @@ def test_transformation_of_tabulated_data_prism(element_type, degree, element_ar
             for d in range(e.value_size):
                 i_slice = i[:, d]
                 j_slice = j[:, d]
-                assert np.allclose(bt[10].dot(i_slice)[start: start + ndofs],
-                                   j_slice[start: start + ndofs])
+                assert np.allclose(
+                    bt[10].dot(i_slice)[start : start + ndofs], j_slice[start : start + ndofs]
+                )
 
     if ndofs != 0:
-        # Check that the 11th transformation undoes the effect of reflecting face 0
+        # Check that the 11th transformation undoes the effect of
+        # reflecting face 0
         reflected_points = np.array([[p[1], p[0], p[2]] for p in points])
         reflected_values = e.tabulate(0, reflected_points)[0]
-
         _J = np.array([[0, 1, 0], [1, 0, 0], [0, 0, 1]])
         J = np.array([_J for p in points])
         detJ = np.array([np.linalg.det(_J) for p in points])
@@ -402,8 +417,9 @@ def test_transformation_of_tabulated_data_prism(element_type, degree, element_ar
             for d in range(e.value_size):
                 i_slice = i[:, d]
                 j_slice = j[:, d]
-                assert np.allclose((bt[11].dot(i_slice))[start: start + ndofs],
-                                   j_slice[start: start + ndofs])
+                assert np.allclose(
+                    (bt[11].dot(i_slice))[start : start + ndofs], j_slice[start : start + ndofs]
+                )
 
 
 @parametrize_over_elements(3, basix.CellType.pyramid)
@@ -412,17 +428,23 @@ def test_transformation_of_tabulated_data_pyramid(element_type, degree, element_
     bt = e.base_transformations()
 
     N = 4
-    points = np.array([[i / N, j / N, k / N]
-                       for i in range(N + 1) for j in range(N + 1 - i) for k in range(N + 1)])
+    points = np.array(
+        [
+            [i / N, j / N, k / N]
+            for i in range(N + 1)
+            for j in range(N + 1 - i)
+            for k in range(N + 1)
+        ]
+    )
     values = e.tabulate(0, points)[0]
 
     start = sum(e.num_entity_dofs[0])
     ndofs = e.num_entity_dofs[1][0]
     if ndofs != 0:
-        # Check that the 0th transformation undoes the effect of reflecting edge 0
+        # Check that the 0th transformation undoes the effect of
+        # reflecting edge 0
         reflected_points = np.array([[1 - p[2] - p[0], p[1], p[2]] for p in points])
         reflected_values = e.tabulate(0, reflected_points)[0]
-
         _J = np.array([[-1, 0, 0], [0, 1, 0], [-1, 0, 1]])
         J = np.array([_J for p in points])
         detJ = np.array([np.linalg.det(_J) for p in points])
@@ -432,16 +454,17 @@ def test_transformation_of_tabulated_data_pyramid(element_type, degree, element_
             for d in range(e.value_size):
                 i_slice = i[:, d]
                 j_slice = j[:, d]
-                assert np.allclose((bt[0].dot(i_slice))[start: start + ndofs],
-                                   j_slice[start: start + ndofs])
+                assert np.allclose(
+                    (bt[0].dot(i_slice))[start : start + ndofs], j_slice[start : start + ndofs]
+                )
 
     start = sum(e.num_entity_dofs[0]) + sum(e.num_entity_dofs[1])
     ndofs = e.num_entity_dofs[2][0]
     if ndofs != 0:
-        # Check that the 8th transformation undoes the effect of rotating face 0
+        # Check that the 8th transformation undoes the effect of
+        # rotating face 0
         rotated_points = np.array([[1 - p[1] - p[2], p[0], p[2]] for p in points])
         rotated_values = e.tabulate(0, rotated_points)[0]
-
         _J = np.array([[0, 1, 0], [-1, 0, 0], [-1, 0, 1]])
         J = np.array([_J for p in points])
         detJ = np.array([np.linalg.det(_J) for p in points])
@@ -451,14 +474,15 @@ def test_transformation_of_tabulated_data_pyramid(element_type, degree, element_
             for d in range(e.value_size):
                 i_slice = i[:, d]
                 j_slice = j[:, d]
-                assert np.allclose(bt[8].dot(i_slice)[start: start + ndofs],
-                                   j_slice[start: start + ndofs])
+                assert np.allclose(
+                    bt[8].dot(i_slice)[start : start + ndofs], j_slice[start : start + ndofs]
+                )
 
     if ndofs != 0:
-        # Check that the 9th transformation undoes the effect of reflecting face 0
+        # Check that the 9th transformation undoes the effect of
+        # reflecting face 0
         reflected_points = np.array([[p[1], p[0], p[2]] for p in points])
         reflected_values = e.tabulate(0, reflected_points)[0]
-
         _J = np.array([[0, 1, 0], [1, 0, 0], [0, 0, 1]])
         J = np.array([_J for p in points])
         detJ = np.array([np.linalg.det(_J) for p in points])
@@ -468,5 +492,102 @@ def test_transformation_of_tabulated_data_pyramid(element_type, degree, element_
             for d in range(e.value_size):
                 i_slice = i[:, d]
                 j_slice = j[:, d]
-                assert np.allclose((bt[9].dot(i_slice))[start: start + ndofs],
-                                   j_slice[start: start + ndofs])
+                assert np.allclose(
+                    (bt[9].dot(i_slice))[start : start + ndofs], j_slice[start : start + ndofs]
+                )
+
+
+@pytest.mark.parametrize(
+    "family, cell_type, degree, args, subentity, results",
+    [
+        (
+            basix.ElementFamily.P,
+            basix.CellType.tetrahedron,
+            1,
+            (),
+            basix.CellType.triangle,
+            {
+                0: [0, 1, 2],
+                2: [1, 2, 0],
+                4: [2, 0, 1],
+                1: [0, 2, 1],
+                3: [1, 0, 2],
+                5: [2, 1, 0],
+            },
+        ),
+        (
+            basix.ElementFamily.P,
+            basix.CellType.tetrahedron,
+            2,
+            (),
+            basix.CellType.triangle,
+            {
+                0: [0, 1, 2, 3, 4, 5],
+                2: [1, 2, 0, 4, 5, 3],
+                4: [2, 0, 1, 5, 3, 4],
+                1: [0, 2, 1, 3, 5, 4],
+                3: [1, 0, 2, 4, 3, 5],
+                5: [2, 1, 0, 5, 4, 3],
+            },
+        ),
+        (
+            basix.ElementFamily.P,
+            basix.CellType.tetrahedron,
+            3,
+            (basix.LagrangeVariant.equispaced,),
+            basix.CellType.triangle,
+            {
+                0: [0, 1, 2, 3, 4, 5, 6, 7, 8],
+                2: [1, 2, 0, 6, 5, 8, 7, 3, 4],
+                4: [2, 0, 1, 7, 8, 4, 3, 6, 5],
+                1: [0, 2, 1, 4, 3, 7, 8, 5, 6],
+                3: [1, 0, 2, 5, 6, 3, 4, 8, 7],
+                5: [2, 1, 0, 8, 7, 6, 5, 4, 3],
+            },
+        ),
+        (
+            basix.ElementFamily.P,
+            basix.CellType.tetrahedron,
+            1,
+            (basix.LagrangeVariant.equispaced,),
+            basix.CellType.interval,
+            {
+                0: [0, 1],
+                1: [1, 0],
+            },
+        ),
+        (
+            basix.ElementFamily.P,
+            basix.CellType.tetrahedron,
+            2,
+            (basix.LagrangeVariant.equispaced,),
+            basix.CellType.interval,
+            {
+                0: [0, 1, 2],
+                1: [1, 0, 2],
+            },
+        ),
+        (
+            basix.ElementFamily.P,
+            basix.CellType.tetrahedron,
+            3,
+            (basix.LagrangeVariant.equispaced,),
+            basix.CellType.interval,
+            {
+                0: [0, 1, 2, 3],
+                1: [1, 0, 3, 2],
+            },
+        ),
+    ],
+)
+def test_permute_subentity_closure(family, cell_type, degree, args, subentity, results):
+    e = basix.create_element(family, cell_type, degree, *args)
+
+    for entity_info, result in results.items():
+        data = np.arange(len(result))
+        data = list(e._e.permute_subentity_closure(data, entity_info, subentity.value))
+        assert result == data
+
+        data = np.array(result)
+        data = list(e._e.permute_subentity_closure_inv(data, entity_info, subentity.value))
+        assert data == list(range(len(result)))
