@@ -2,11 +2,13 @@
 # FEniCS Project
 # SPDX-License-Identifier: MIT
 
-import numpy as np
-import basix
-import pytest
 from itertools import product
-import numpy
+
+import numpy as np
+import pytest
+
+import basix
+
 from .utils import parametrize_over_elements
 
 
@@ -22,22 +24,22 @@ def tensor_product(*data):
 
 @parametrize_over_elements(4)
 def test_tensor_product_factorisation(cell_type, degree, element_type, element_args):
-    element = basix.create_element(element_type, cell_type, degree, *element_args)
-    tdim = len(basix.topology(cell_type)) - 1
-
-    # These elements should have a factorisation
-    if cell_type in [
-        basix.CellType.quadrilateral, basix.CellType.hexahedron
-    ] and element_type in [
-        basix.ElementFamily.P
-    ] and basix.LagrangeVariant.equispaced in element_args:
-        assert element.has_tensor_product_factorisation
-
-    if not element.has_tensor_product_factorisation:
-        with pytest.raises(RuntimeError):
-            element.get_tensor_product_representation()
+    try:
+        element = basix.create_tp_element(element_type, cell_type, degree, *element_args)
+    except RuntimeError:
+        # These elements should have a factorisation
+        if (
+            cell_type in [basix.CellType.quadrilateral, basix.CellType.hexahedron]
+            and element_type in [basix.ElementFamily.P]
+            and basix.LagrangeVariant.equispaced in element_args
+        ):
+            raise RuntimeError("Could not create tensor product element")
         pytest.skip()
 
+    assert element.has_tensor_product_factorisation
+
+    tdim = len(basix.topology(cell_type)) - 1
+
     factors = element.get_tensor_product_representation()
 
     lattice = basix.create_lattice(cell_type, 1, basix.LatticeType.equispaced, True)
@@ -48,15 +50,13 @@ def test_tensor_product_factorisation(cell_type, degree, element_type, element_a
             if sum(ds) > 1:
                 continue
             deriv = basix.index(*ds)
-            values1 = tab1[deriv, i, :, :]
+            values1 = tab1[deriv, i, :, 0]
 
-            values2 = np.empty((element.dim, 1))
-            for fs, perm in factors:
-                evals = [e.tabulate(d, p.reshape(1, -1))[d, 0, :, 0] for e, p, d in zip(fs, point, ds)]
-                tab2 = tensor_product(*evals)
-                for a, b in enumerate(perm):
-                    if b != -1:
-                        values2[b, 0] = tab2[a]
+            for fs in factors:
+                evals = [
+                    e.tabulate(d, p.reshape(1, -1))[d, 0, :, 0] for e, p, d in zip(fs, point, ds)
+                ]
+                values2 = tensor_product(*evals)
             assert np.allclose(values1, values2)
 
 
@@ -64,24 +64,25 @@ def test_tensor_product_factorisation(cell_type, degree, element_type, element_a
 def test_tensor_product_factorisation_quadrilateral(degree):
     P = degree
     cell_type = basix.CellType.quadrilateral
-    element = basix.create_element(basix.ElementFamily.P, cell_type,
-                                   P, basix.LagrangeVariant.gll_warped)
+    element = basix.create_tp_element(
+        basix.ElementFamily.P, cell_type, P, basix.LagrangeVariant.gll_warped
+    )
     factors = element.get_tensor_product_representation()[0]
+    # FIXME: This test assumes all factors formed by a single element
+    element0 = factors[0]
 
     # Quadrature degree
     Q = 2 * P + 2
-    points, w = basix.make_quadrature(basix.QuadratureType.Default, cell_type, Q)
+    points, w = basix.make_quadrature(cell_type, Q)
+
     data = element.tabulate(1, points)
     dphi_x = data[1, :, :, 0]
     dphi_y = data[2, :, :, 0]
 
-    assert points.shape[0] == (P+2) * (P+2)
+    assert points.shape[0] == (P + 2) * (P + 2)
 
-    # FIXME: This test assumes all factors formed by a single element
-    perm = factors[1]
-    element0 = factors[0][0]
     cell1d = element0.cell_type
-    points, w = basix.make_quadrature(basix.QuadratureType.Default, cell1d, Q)
+    points, _ = basix.make_quadrature(cell1d, Q)
     data = element0.tabulate(1, points)
     phi0 = data[0, :, :, 0]
     dphi0 = data[1, :, :, 0]
@@ -92,49 +93,51 @@ def test_tensor_product_factorisation_quadrilateral(degree):
     Nq = P + 2
 
     # Compute derivative of basis function in the x direction
-    dphi_tensor = numpy.zeros([Nq, Nq, Nd, Nd])
+    dphi_tensor = np.zeros([Nq, Nq, Nd, Nd])
     for q0 in range(Nq):
         for q1 in range(Nq):
             for i0 in range(Nd):
                 for i1 in range(Nd):
-                    dphi_tensor[q0, q1, i0, i1] = dphi0[q0, i0]*phi0[q1, i1]
-    dphi_tensor = dphi_tensor.reshape([Nq*Nq, Nd*Nd])
-    assert numpy.allclose(dphi_x[:, perm], dphi_tensor)
+                    dphi_tensor[q0, q1, i0, i1] = dphi0[q0, i0] * phi0[q1, i1]
+    dphi_tensor = dphi_tensor.reshape([Nq * Nq, Nd * Nd])
+    assert np.allclose(dphi_x, dphi_tensor)
 
     # Compute derivative of basis function in the y direction
-    dphi_tensor = numpy.zeros([Nq, Nq, Nd, Nd])
+    dphi_tensor = np.zeros([Nq, Nq, Nd, Nd])
     for q0 in range(Nq):
         for q1 in range(Nq):
             for i0 in range(Nd):
                 for i1 in range(Nd):
-                    dphi_tensor[q0, q1, i0, i1] = phi0[q0, i0]*dphi0[q1, i1]
+                    dphi_tensor[q0, q1, i0, i1] = phi0[q0, i0] * dphi0[q1, i1]
 
-    dphi_tensor = dphi_tensor.reshape([Nq*Nq, Nd*Nd])
-    assert numpy.allclose(dphi_y[:, perm], dphi_tensor)
+    dphi_tensor = dphi_tensor.reshape([Nq * Nq, Nd * Nd])
+    assert np.allclose(dphi_y, dphi_tensor)
 
 
 @pytest.mark.parametrize("degree", range(1, 6))
 def test_tensor_product_factorisation_hexahedron(degree):
     P = degree
-    element = basix.create_element(basix.ElementFamily.P, basix.CellType.hexahedron,
-                                   P, basix.LagrangeVariant.gll_warped)
+    element = basix.create_tp_element(
+        basix.ElementFamily.P, basix.CellType.hexahedron, P, basix.LagrangeVariant.gll_warped
+    )
     factors = element.get_tensor_product_representation()[0]
 
     # Quadrature degree
     Q = 2 * P + 2
-    points, w = basix.make_quadrature(basix.QuadratureType.Default, basix.CellType.hexahedron, Q)
+    points, _ = basix.make_quadrature(basix.CellType.hexahedron, Q)
+
+    # FIXME: This test assumes all factors formed by a single element
+    element0 = factors[0]
+
     data = element.tabulate(1, points)
     dphi_x = data[1, :, :, 0]
     dphi_y = data[2, :, :, 0]
     dphi_z = data[3, :, :, 0]
 
-    assert points.shape[0] == (P+2) * (P+2) * (P+2)
+    assert points.shape[0] == (P + 2) * (P + 2) * (P + 2)
 
-    # FIXME: This test assumes all factors formed by a single element
-    perm = factors[1]
-    element0 = factors[0][0]
     cell1d = element0.cell_type
-    points, w = basix.make_quadrature(basix.QuadratureType.Default, cell1d, Q)
+    points, w = basix.make_quadrature(cell1d, Q)
     data = element0.tabulate(1, points)
     phi0 = data[0, :, :, 0]
     dphi0 = data[1, :, :, 0]
@@ -145,40 +148,70 @@ def test_tensor_product_factorisation_hexahedron(degree):
     Nq = P + 2
 
     # Compute derivative of basis function in the x direction
-    dphi_tensor = numpy.zeros([Nq, Nq, Nq, Nd, Nd, Nd])
+    dphi_tensor = np.zeros([Nq, Nq, Nq, Nd, Nd, Nd])
     for q0 in range(Nq):
         for q1 in range(Nq):
             for q2 in range(Nq):
                 for i0 in range(Nd):
                     for i1 in range(Nd):
                         for i2 in range(Nd):
-                            dphi_tensor[q0, q1, q2, i0, i1, i2] = dphi0[q0, i0]*phi0[q1, i1]*phi0[q2, i2]
+                            dphi_tensor[q0, q1, q2, i0, i1, i2] = (
+                                dphi0[q0, i0] * phi0[q1, i1] * phi0[q2, i2]
+                            )
 
-    dphi_tensor = dphi_tensor.reshape([Nq*Nq*Nq, Nd*Nd*Nd])
-    assert numpy.allclose(dphi_x[:, perm], dphi_tensor)
+    dphi_tensor = dphi_tensor.reshape([Nq * Nq * Nq, Nd * Nd * Nd])
+    assert np.allclose(dphi_x, dphi_tensor)
 
     # Compute derivative of basis function in the y direction
-    dphi_tensor = numpy.zeros([Nq, Nq, Nq, Nd, Nd, Nd])
+    dphi_tensor = np.zeros([Nq, Nq, Nq, Nd, Nd, Nd])
     for q0 in range(Nq):
         for q1 in range(Nq):
             for q2 in range(Nq):
                 for i0 in range(Nd):
                     for i1 in range(Nd):
                         for i2 in range(Nd):
-                            dphi_tensor[q0, q1, q2, i0, i1, i2] = phi0[q0, i0]*dphi0[q1, i1]*phi0[q2, i2]
+                            dphi_tensor[q0, q1, q2, i0, i1, i2] = (
+                                phi0[q0, i0] * dphi0[q1, i1] * phi0[q2, i2]
+                            )
 
-    dphi_tensor = dphi_tensor.reshape([Nq*Nq*Nq, Nd*Nd*Nd])
-    assert numpy.allclose(dphi_y[:, perm], dphi_tensor)
+    dphi_tensor = dphi_tensor.reshape([Nq * Nq * Nq, Nd * Nd * Nd])
+    assert np.allclose(dphi_y, dphi_tensor)
 
     # Compute the derivative of basis function in the z direction
-    dphi_tensor = numpy.zeros([Nq, Nq, Nq, Nd, Nd, Nd])
+    dphi_tensor = np.zeros([Nq, Nq, Nq, Nd, Nd, Nd])
     for q0 in range(Nq):
         for q1 in range(Nq):
             for q2 in range(Nq):
                 for i0 in range(Nd):
                     for i1 in range(Nd):
                         for i2 in range(Nd):
-                            dphi_tensor[q0, q1, q2, i0, i1, i2] = phi0[q0, i0]*phi0[q1, i1]*dphi0[q2, i2]
+                            dphi_tensor[q0, q1, q2, i0, i1, i2] = (
+                                phi0[q0, i0] * phi0[q1, i1] * dphi0[q2, i2]
+                            )
 
-    dphi_tensor = dphi_tensor.reshape([Nq*Nq*Nq, Nd*Nd*Nd])
-    assert numpy.allclose(dphi_z[:, perm], dphi_tensor)
+    dphi_tensor = dphi_tensor.reshape([Nq * Nq * Nq, Nd * Nd * Nd])
+    assert np.allclose(dphi_z, dphi_tensor)
+
+
+@pytest.mark.parametrize(
+    "cell_type",
+    [
+        basix.CellType.quadrilateral,
+        basix.CellType.hexahedron,
+    ],
+)
+@pytest.mark.parametrize(
+    "family, args",
+    [
+        (basix.ElementFamily.P, (basix.LagrangeVariant.equispaced,)),
+        (basix.ElementFamily.P, (basix.LagrangeVariant.gll_warped,)),
+    ],
+)
+@pytest.mark.parametrize("degree", range(1, 5))
+def test_dof_ordering(cell_type, family, args, degree):
+    e = basix.create_tp_element(family, cell_type, degree, *args)
+    perm = e.dof_ordering
+    e2 = basix.create_element(family, cell_type, degree, *args, dof_ordering=perm)
+    assert e2.has_tensor_product_factorisation
+
+    assert perm == basix.finite_element.tp_dof_ordering(family, cell_type, degree, *args)
